% coma -- maintain widget state.
%
%      [...] = coma(command, ...)
%
% Coma is a tool to maintain the state of a Matlab graphical
% user interface between separate application invocations.
%
% First argument COMMAND is a string.  Below is a list of all
% public commands together with their meanings.
%
%   help              Display help text.
%   handle            Return function handle.
%   figure            Return figure handle.
%   event             Coma event handler.
%   open              Load saved configuration.
%   close             Clear current configuration.
%   save              Save current configuration.
%   exit              Quit application.
%   backup            Backup current configuration.
%   restore           Restore backed-up configuration.
%   read              Read configuration from file.
%   write             Write configuration to file.
%   setup             Install Coma in application.
%   factory           Create factory defaults for widgets.
%   title             Maintain corporate window title.
%   dirty             Handle dirty flag.
%   enable            Handle enable flag.
%   visible           Handle visibility flag.
%   tick              Handle tick mark.
%
% Enter 'coma help COMMAND' to display the detailed help text
% for a command.
%
% See also: mtk

%% coma.m --- the preceding comment is the documentation string.

% Copyright (C) 2008 Ralph Schleicher

% This file is part of MTk, the Matlab widget toolkit.

% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions
% are met:
%
%    * Redistributions of source code must retain the above copyright
%      notice, this list of conditions and the following disclaimer.
%
%    * Redistributions in binary form must reproduce the above copyright
%      notice, this list of conditions and the following disclaimer in
%      the documentation and/or other materials provided with the
%      distribution.
%
%    * Neither the name of the copyright holder nor the names of its
%      contributors may be used to endorse or promote products derived
%      from this software without specific prior written permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
% "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
% FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
% COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
% INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
% CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
% LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
% ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.

%% Code:

% Program entry point.
function varargout = coma(varargin)

  try
    % Call subfunction quickly.
    f = str2func(['coma_', varargin{1}]);
    if nargout == 0
      f(varargin{2:end});
    else
      varargout = cell(1, nargout);
      [varargout{:}] = f(varargin{2:end});
    end
  catch
    if exist('rethrow', 'builtin')
      rethrow(mtk('suberr', lasterror, 'coma_', varargin));
    else
      error(mtk('suberr', lasterr, 'coma_', varargin));
    end
  end

% coma/help -- display help text.
%
%      coma('help', topic)
%      coma('help')
%
% Display the help text for a Coma command.
%
% If the first argument TOPIC is omitted, display the help text for
% the gateway command.
function coma_help(topic)

  if nargin < 1
    topic = [];
  end

  mtk('help', 'coma', topic, mfilename);

% coma/handle -- get function handle.
%
%      h = coma('handle', command, ...)
%
% Return a function handle to the specified Coma command.
%
% Optional first argument COMMAND is the name of a Coma command.
% Remaining arguments are subcommand names.
%
% Return value H is the function handle of the Coma command.
function h = coma_handle(varargin)

  % Build function name.
  tem = {'_'};

  str = [tem(ones(1, nargin)); varargin];
  str = ['coma', str{:}];

  h = str2func(str);

% coma/figure -- get figure handle.
%
%      fig = coma('figure', obj)
%
% Return parent figure handle of a widget.
%
% Argument OBJ is a Handle Graphics object.
%
% This command is more reliable than 'gcf' and 'gcbf' because it
% works in conjunction with explicit invocations of call-back
% functions (a.k.a. event handlers) and it has no side effects.
%
% Return value is the parent figure handle.
%
% See also: gcf, gcbf
function fig = coma_figure(obj)

  fig = obj;

  while 1
    tem = get(fig, 'Parent');
    if tem == 0
      return;
    end
    fig = tem;
  end

% coma/event -- Coma event handler.
%
%      coma('event', obj, ev, command, ...)
%
% Process a call-back event.
%
% First argument COMMAND is a string.  Below is a list of all
% commands together with their meanings.
%
%   open              Load saved configuration.
%   close             Clear current configuration.
%   save              Save current configuration.
%   save as           Save current configuration to a new file.
%   exit              Quit application.
%
% The 'save as' command can also be spelled 'save_as' or 'save-as'
% and it is equal to the 'save' command with an empty file name
% argument.  Enter 'coma help COMMAND' to display the detailed
% help text for a command and the number and meaning of any
% additional arguments.
%
% This function is designed to work as a call-back handler for
% Handle Graphics objects.  For example,
%
%      fun = coma('handle', 'event');
%      set(obj, 'Callback', {fun, 'open'});
%      set(fig, 'CloseRequestFcn', {fun, 'exit'});
function coma_event(obj, ev, command, varargin)

  % Call-back figure.
  fig = coma_figure(obj);

  try
    switch command
     case 'open'
      coma_open(fig, varargin{:});
     case 'close'
      coma_close(fig, varargin{:});
     case 'save'
      coma_save(fig, varargin{:});
     case {'save as', 'save_as', 'save-as'}
      coma_save(fig, [], varargin{:});
     case 'exit'
      coma_exit(fig, varargin{:});
     otherwise
      error('Unknown call-back command');
    end
  catch
    mtk('error', coma_title(fig, 'Error'), lasterr);
  end

% coma/open -- load saved configuration.
%
%      coma('open', fig, file)
%
% First argument FIG is the figure window handle.
% Optional second argument FILE is the configuration file name.
%  If the FILE argument is empty, the effect is equal to the
%  'close' command.  If the FILE argument is omitted, open a
%  file selection dialog box.
%
% The 'open' command runs the 'ReadFunction' to actually read the
% configuration from the file FILE.  If the current configuration
% has been modified, the user is prompted whether or not to discard
% the changes.  After the new configuration has been applied to the
% application, the figure's dirty flag is cleared and the window
% title is updated.
function coma_open(fig, file)

  app = getappdata(fig, 'coma_private');

  if nargin < 2
    file = getappdata(fig, 'coma_config');
    if isempty(file)
      dir = app.Directory;
    else
      dir = fileparts(file);
    end
    file = mtk('filesel', ...
               'Style', 'open', ...
               'WindowTitle', coma_title(fig, 'Open'), ...
               'Directory', dir, ...
               'DefaultFileName', file, ...
               'Filter', app.FileFilter);
    if isempty(file)
      return;
    end
  elseif isempty(file)
    coma_close(fig);
    return;
  end

  % Read configuration file.
  config = coma_run_hook(app.ReadFunction, file);

  % Attempt to apply the configuration.
  if coma_keep(fig)
    return;
  end

  backup = coma_backup(fig);

  try
    coma_restore(fig, config, 1);
    coma_dirty(fig, 0);
  catch
    coma_restore(fig, backup);
    error('Invalid configuration file');
  end

  % Save configuration file name.
  setappdata(fig, 'coma_config', file);
  setappdata(fig, 'coma_tag', coma_filename(file));
  coma_title(fig);

% coma/close -- clear current configuration.
%
%      coma('close', fig)
%
% Argument FIG is the figure window handle.
%
% The 'close' command resets the application to its factory
% defaults.  If the current configuration has been modified, the
% user is prompted whether or not to discard the changes.  After
% the factory configuration has been applied to the application,
% the figure's dirty flag is cleared and the window title is
% updated.
function coma_close(fig)

  if coma_keep(fig)
    return;
  end

  app = getappdata(fig, 'coma_private');

  coma_restore(fig, app.FactoryDefaults);
  coma_dirty(fig, 0);

  % Clear configuration file name.
  setappdata(fig, 'coma_config', []);
  setappdata(fig, 'coma_tag', []);
  coma_title(fig);

% coma/save -- save current configuration.
%
%      coma('save', fig, file)
%
% First argument FIG is the figure window handle.
% Optional second argument FILE is the configuration file name.
%  If the FILE argument is omitted, use the current configuration
%  file name.  If no configuration file name is present or if the
%  FILE argument is empty, open a file selection dialog box.
%
% The 'save' command first runs the 'ValidateHook' to validate
% the current configuration.  Then it runs the 'WriteFunction'
% to actually write the configuration to the file FILE.  After
% the configuration has been saved, the figure's dirty flag is
% cleared and the window title is updated.
function coma_save(fig, file)

  app = getappdata(fig, 'coma_private');

  if ~ isempty(app.ValidateHook)
    coma_run_hook(app.ValidateHook, fig);
  end

  if nargin < 2
    file = getappdata(fig, 'coma_config');
  end
  if isempty(file)
    file = mtk('filesel', ...
               'Style', 'save', ...
               'WindowTitle', coma_title(fig, 'Save'), ...
               'Directory', app.Directory, ...
               'Filter', app.FileFilter);
    if isempty(file)
      return;
    end
  end

  % Save the current configuration.
  coma_run_hook(app.WriteFunction, file, coma_backup(fig, 1));
  coma_dirty(fig, 0);

  % Save configuration file name.
  setappdata(fig, 'coma_config', file);
  setappdata(fig, 'coma_tag', coma_filename(file));
  coma_title(fig);

% coma/exit -- quit application.
%
%      coma('exit', fig)
%
% Argument FIG is the figure window handle.
%
% The 'exit' command only deletes the figure window if the
% application is clean, that is the figure's dirty flag is
% not set.  If the current configuration has been modified,
% the user is prompted whether or not to discard the changes.
function coma_exit(fig)

  if coma_keep(fig)
    return;
  end

  delete(fig);

% coma/keep -- attempt to save a modified configuration.
%
%      p = coma('keep', fig)
%
% Check whether or not the current configuration is save.
%
% Argument FIG is the figure window handle.
%
% The 'keep' command checks if the current configuration has been
% modified.  If so, the user is prompted whether or not to discard
% the changes.
%
% Return value is true if the figure shall be kept as it is.
function p = coma_keep(fig)

  if coma_dirty(fig)
    if ~ mtk('question', coma_title(fig, 'Question'), ...
             'Modified entries exist; discard changes?')
      coma_save(fig);
      if coma_dirty(fig)
        p = 1;
      end
    end
  end

  p = 0;

% coma/backup -- backup current configuration.
%
%      s = coma('backup', fig, terse)
%
% Return the current configuration.
%
% First argument FIG is the figure window handle.
% If optional second argument TERSE is true, the configuration will
%  be saved in a file.  Therefore, 'BackupHook' is run, too.  This
%  allows the user to clear values which are not persistent.
function s = coma_backup(fig, terse)

  if nargin < 2 || isempty(terse)
    terse = 0;
  end

  app = getappdata(fig, 'coma_private');

  s = app.FactoryDefaults;
  if ~ isstruct(s)
    s = [];
    return;
  end

  w = getappdata(fig, 'coma_widgets');
  if isstruct(w)
    if ~ isempty(app.PreBackupHook)
      coma_run_hook(app.PreBackupHook, fig);
    end

    tag = fieldnames(s);
    for i = 1:numel(tag)
      if isfield(w, tag{i})
        prop = fieldnames(s.(tag{i}));
        val = get(w.(tag{i}), prop);
        s.(tag{i}) = cell2struct(val, prop, 2);
      end
    end

    if ~ isempty(app.PostBackupHook)
      coma_run_hook(app.PostBackupHook, fig);
    end
  end

  if terse
    if ~ isempty(app.BackupHook)
      s = coma_run_hook(app.BackupHook, fig, s);
      if ~ isstruct(s)
        s = [];
        return;
      end
    end
  end

% coma/restore -- restore backed-up configuration.
%
%      coma('restore', fig, s, terse)
%
% Apply a configuration.
%
% First argument FIG is the figure window handle.
% Second argument S is a configuration.
% If optional third argument TERSE is true, the configuration has
%  been loaded from a file.  Therefore, 'RestoreHook' is run, too.
%  This allows the user to set values which are not persistent.
function coma_restore(fig, s, terse)

  if nargin < 3 || isempty(terse)
    terse = 0;
  end

  app = getappdata(fig, 'coma_private');

  if ~ isstruct(s)
    return;
  end

  if terse
    if ~ isempty(app.RestoreHook)
      s = coma_run_hook(app.RestoreHook, fig, s);
      if ~ isstruct(s)
        return;
      end
    end
  end

  w = getappdata(fig, 'coma_widgets');
  if isstruct(w)
    if ~ isempty(app.PreRestoreHook)
      coma_run_hook(app.PreRestoreHook, fig);
    end

    tag = fieldnames(s);
    for i = 1:numel(tag)
      if isfield(w, tag{i})
        set(w.(tag{i}), s.(tag{i}));
      end
    end

    if ~ isempty(app.PostRestoreHook)
      coma_run_hook(app.PostRestoreHook, fig);
    end
  end

% coma/read -- load configuration from file.
%
%      s = coma('read', filename)
%
% Load configuration from file.
%
% Argument FILENAME is the file name.
%
% Return value is the configuration.
function config = coma_read(filename)

  % Load MAT-file.
  load('-mat', filename, 'config');

% coma/write -- save configuration to file.
%
%      coma('write', filename, s)
%
% Save configuration to file.
%
% First argument FILENAME is the file name.
% Second argument S is the configuration.
function coma_write(filename, config)

  % Save MAT-file.
  if sscanf(version, '%d') > 6
    save(filename, 'config', '-v6');
  else
    save(filename, 'config', '-mat');
  end

% coma/setup -- setup figure window.
%
%      coma('setup', fig, ...)
%
% Install Coma in application.
%
% First argument FIG is the figure window handle.
% Remaining arguments are property/value pairs.  Below is a list of
%  all properties together with their meanings.
%
% FactoryDefaults
%      The factory configuration, that is the configuration of
%      a new instance of the application.  Value is a structure.
%      See the 'coma/factory' command, for more details.
%
% Directory
%      The default directory when opening and saving configuration
%      files interactively, that is via a file selection dialog
%      box.  This property only has an effect if the directory
%      exist in advance.  Value is a string.
%
% FileExtension
%      The file extension for configuration files, default '.mat'.
%      Value is a string.
%
% FileFilter
%      The file name filter specification for configuration files.
%      Default is to add an entry for the file extension.  Value is
%      a cell array of strings.  See 'mtk/filesel', 'uigetfile', or
%      'uiputfile', for more details.
%
% PreBackupHook(fig)
%      Call-back function to be run before a configuration is
%      backed up.  Argument FIG is the figure window handle.
%
% PostBackupHook(fig)
%      Call-back function to be run after a configuration has been
%      backed up.  Argument FIG is the figure window handle.
%
% s = BackupHook(fig, s)
%      Call-back function to be run after a configuration has been
%      backed up.  This function will only be run for terse backups.
%      Argument FIG is the figure window handle.  Second argument S
%      is the backed up configuration.  Return value S is the terse
%      configuration, that means elements not to be saved to a file
%      are either removed or cleared.
%
% s = RestoreHook(fig, s)
%      Call-back function to be run before a configuration is
%      restored.  This function will only be run for terse restores
%      (see 'BackupHook' above).  Argument FIG is the figure window
%      handle.  Second argument S is the terse configuration.
%      Return value S is the configuration to be restored.
%
% PreRestoreHook(fig)
%      Call-back function to be run before a configuration is
%      restored.  Argument FIG is the figure window handle.
%
% PostRestoreHook(fig)
%      Call-back function to be run after a configuration has been
%      restored.  Argument FIG is the figure window handle.
%
% ValidateHook(fig)
%      Call-back function to be run before a configuration is saved
%      to file.  Argument FIG is the figure window handle.  Function
%      should signal an error if the figure window contains invalid
%      user input.
%
% s = ReadFunction(file)
%      Function to read a configuration from a file.  Default is
%      the 'coma/read' command.  Argument FILE is the file name.
%      Return value is the configuration.
%
% WriteFunction(file, s)
%      Function to write a configuration to a file.  Default is
%      the 'coma/write' command.  First argument FILE is the file
%      name.  Second argument S is the configuration.
%
% Coma sets the following figure application data.
%
% coma_title
%      Corporate window title (a string).
%
% coma_widgets
%      Widget handles (a 'guihandles' structure).
%
% coma_handles
%      Widget handles (a vector).
%
% coma_tag
%      Current configuration identifier (a string).  Usually
%      the base name of the configuration file name.
%
% coma_config
%      Configuration file name (a string).
%
% coma_dirty
%      Whether or not the current configuration has been modified.
function coma_setup(fig, varargin)

  % Check arguments.
  if nargin < 1
    error(nargchk(1, 1, 0));
  end

  % Parse property/value pairs.
  prop = struct('FactoryDefaults', [], ...
                'Directory', [], ...
                'FileExtension', [], ...
                'FileFilter', [], ...
                'PreBackupHook', [], ...
                'PostBackupHook', [], ...
                'BackupHook', [], ...
                'RestoreHook', [], ...
                'PreRestoreHook', [], ...
                'PostRestoreHook', [], ...
                'ValidateHook', [], ...
                'ReadFunction', @coma_read, ...
                'WriteFunction', @coma_write);

  n = numel(varargin);
  if rem(n, 2) ~= 0
    error('Wrong number of arguments');
  end

  name = fieldnames(prop);

  for j = 1:2:n
    arg = varargin{j};
    if ~ mtk('stringp', arg)
      error('Invalid property name');
    end
    k = find(strcmpi(arg, name));
    if isempty(k)
      error('Unknown property ''%s''', arg);
    end
    prop = setfield(prop, name{k}, varargin{j + 1});
  end

  val = prop.FactoryDefaults;
  if ~ ((isnumeric(val) && isempty(val)) ...
        || (isstruct(val) && numel(val) == 1))
    error('Factory defaults has to be a structure');
  end

  val = prop.Directory;
  if ~ ((isnumeric(val) && isempty(val)) ...
        || mtk('stringp', val))
    error('Default directory has to be a string');
  end

  val = prop.FileExtension;
  if ~ ((isnumeric(val) && isempty(val)) ...
        || mtk('stringp', val))
    error('File extension has to be a string');
  end

  val = prop.FileFilter;
  if ~ ((isnumeric(val) && isempty(val)) ...
        || (iscellstr(val) && ndims(val) == 2 && size(val, 1) >= 1 && size(val, 2) == 2))
    error('File filter has to be a cell matrix of strings');
  end

  val = prop.PreBackupHook;
  if ~ ((isnumeric(val) && isempty(val)) ...
        || coma_hookp(val))
    error('Pre-backup hook has to be a call-back function');
  end

  val = prop.PostBackupHook;
  if ~ ((isnumeric(val) && isempty(val)) ...
        || coma_hookp(val))
    error('Post-backup hook has to be a call-back function');
  end

  val = prop.BackupHook;
  if ~ ((isnumeric(val) && isempty(val)) ...
        || coma_hookp(val))
    error('Backup hook has to be a call-back function');
  end

  val = prop.RestoreHook;
  if ~ ((isnumeric(val) && isempty(val)) ...
        || coma_hookp(val))
    error('Restore hook has to be a call-back function');
  end

  val = prop.PreRestoreHook;
  if ~ ((isnumeric(val) && isempty(val)) ...
        || coma_hookp(val))
    error('Pre-restore hook has to be a call-back function');
  end

  val = prop.PostRestoreHook;
  if ~ ((isnumeric(val) && isempty(val)) ...
        || coma_hookp(val))
    error('Post-restore hook has to be a call-back function');
  end

  val = prop.ValidateHook;
  if ~ ((isnumeric(val) && isempty(val)) ...
        || coma_hookp(val))
    error('Validation hook has to be a call-back function');
  end

  val = prop.ReadFunction;
  if ~ coma_hookp(val)
    error('Read function has to be a call-back function');
  end

  val = prop.WriteFunction;
  if ~ coma_hookp(val)
    error('Write function has to be a call-back function');
  end

  % Fill in defaults.
  if isnumeric(prop.FileFilter)
    if isnumeric(prop.FileExtension)
      prop.FileExtension = '.mat';
    end
    prop.FileFilter = {['*', prop.FileExtension], 'Configuration files'};
  end

  % Gather widget handles.
  w = guihandles(fig);
  if isempty(w)
    h = [];
  else
    tem = struct2cell(w);
    h = [tem{:}];
  end

  setappdata(fig, 'coma_widgets', w);
  setappdata(fig, 'coma_handles', h);

  % Setup configuration control.
  setappdata(fig, 'coma_private', prop);
  setappdata(fig, 'coma_title', get(fig, 'Name'));
  setappdata(fig, 'coma_tag', []);
  setappdata(fig, 'coma_config', []);
  setappdata(fig, 'coma_dirty', 0);

  % Load factory defaults.
  coma_restore(fig, prop.FactoryDefaults);

% coma/factory -- create factory defaults for widgets.
%
%      s = coma('factory', style, ...)
%
% Remaining arguments are property/value pairs.
% Alternative arguments:
%
%      s = coma('factory', 'togglebutton', Value, UserData)
%      s = coma('factory', 'radiobutton', Value, UserData)
%      s = coma('factory', 'checkbox', Value, UserData)
%      s = coma('factory', 'edit', String) % UserData = String
%      s = coma('factory', 'edit', UserData) % String = mat2str(UserData)
%      s = coma('factory', 'edit', String, UserData)
%      s = coma('factory', 'listbox', Value, String, UserData)
%      s = coma('factory', 'popupmenu', Value, UserData)
function s = coma_factory(style, varargin)

  if nargin < 1
    error(nargchk(1, intmax, nargin));
  end

  if ~ mtk('stringp', style)
    error('Invalid argument');
  end

  switch style
   case 'axes'
    s = struct;
   case 'table'
    s = struct;
   case 'panel'
    s = struct;
   case 'pushbutton'
    s = struct;
   case 'togglebutton'
    s = struct('Min', 0, 'Max', 1, 'Value', 0);
   case 'radiobutton'
    s = struct('Min', 0, 'Max', 1, 'Value', 0);
   case 'checkbox'
    s = struct('Min', 0, 'Max', 1, 'Value', 0);
   case 'edit'
    s = struct('String', '');
   case 'text'
    s = struct;
   case 'slider'
    s = struct('Min', 0, 'Max', 1, 'Value', 0, 'SliderStep', [0.01, 0.1]);
   case 'frame'
    s = struct;
   case 'listbox'
    s = struct('Value', 1, 'ListboxTop', 1, 'String', {{}});
   case 'popupmenu'
    s = struct('Value', 1);
   otherwise
    error('Unknown widget style');
  end

  % Optionally parse property/value pairs.
  n = numel(varargin);
  if n == 0
    return;
  end

  persistent prop
  if isempty(prop)
    name = char(fieldnames(get(0, 'FactoryAxes')));
    name(:, 1:numel('FactoryAxes')) = [];
    name = cellstr(name);
    prop.axes.name = name;
    prop.axes.names = lower(name);

    try
      name = char(fieldnames(get(0, 'FactoryUITable')));
      name(:, 1:numel('FactoryUITable')) = [];
      name = cellstr(name);
      prop.uitable.name = name;
      prop.uitable.names = lower(name);
    catch
      % Can't resolve object type in default property 'UITable'.
      prop.uitable.name = {};
      prop.uitable.names = {};
    end

    name = char(fieldnames(get(0, 'FactoryUIPanel')));
    name(:, 1:numel('FactoryUIPanel')) = [];
    name = cellstr(name);
    prop.uipanel.name = name;
    prop.uipanel.names = lower(name);

    name = char(fieldnames(get(0, 'FactoryUIControl')));
    name(:, 1:numel('FactoryUIControl')) = [];
    name = cellstr(name);
    prop.uicontrol.name = name;
    prop.uicontrol.names = lower(name);
  end

  switch style
   case 'axes'
    name = prop.axes.name;
    names = prop.axes.names;
   case 'table'
    name = prop.uitable.name;
    names = prop.uitable.names;
   case 'panel'
    name = prop.uipanel.name;
    names = prop.uipanel.names;
   otherwise
    name = prop.uicontrol.name;
    names = prop.uicontrol.names;
  end

  capture = [];

  for i = 1:2:n
    if ~ mtk('stringp', varargin{i})
      break;
    end

    ind = find(strcmp(lower(varargin{i}), names));
    if isempty(ind)
      if i == 1
        % Try alternative arguments.
        break;
      else
        error('Unknown property');
      end
    end

    if i == n
      if i == 1
        % Likewise.
        break;
      else
        error('Incomplete property/value pair');
      end
    end

    s.(name{ind}) = varargin{i + 1};
    capture = [capture, i, i + 1];
  end

  varargin(capture) = [];

  % Optionally parse alternative arguments.
  n = numel(varargin);
  if n == 0
    return;
  end

  arg = varargin{1};
  switch style
   case {'togglebutton', 'radiobutton', 'checkbox'}
    if ~ isempty(arg)
      if arg
        s.Value = 1;
      end
    end
    varargin(1) = [];
   case 'edit'
    if (isnumeric(arg) || islogical(arg)) && ndims(arg) == 2
      str = mat2str(arg);
      if ~ isa(arg, 'double')
        str = [class(arg), '(', str, ')'];
      end
      s.String = str;
      s.UserData = arg;
      varargin(1) = [];
    elseif ischar(arg) || iscellstr(arg)
      s.String = arg;
      s.UserData = arg;
      varargin(1) = [];
    end
   case 'listbox'
    if mtk('integerp', arg) && arg > 0
      s.Value = arg;
      varargin(1) = [];
    end
    n = numel(varargin);
    if n > 0
      arg = varargin{1};
      if ischar(arg) || iscellstr(arg)
        s.String = arg;
        varargin(1) = [];
      end
    end
   case 'popupmenu'
    if mtk('integerp', arg) && arg > 0
      s.Value = arg;
      varargin(1) = [];
    end
  end

  n = numel(varargin);
  if n > 1
    error(nargchk(0, 1, n));
  end
  if n > 0
    s.UserData = varargin{1};
  end

% coma/title -- return corporate window title.
%
%      str = coma('title', fig, ext)
%
% Return corporate window title.
%
% First argument FIG is a figure window handle.
% Optional second argument EXT is the window title extension.
%
% Return value STR is a string of the form
%
%      'TITLE [TAG] - EXT'
%
% where TITLE is the application's native window title, TAG is the
% current configuration identifier, and EXT is the window title
% extension.
%
% If the return value is omitted, the window title of FIG will be
% modified.
%
% See also: coma/setup
function str = coma_title(fig, ext)

  str = getappdata(fig, 'coma_title');

  tag = getappdata(fig, 'coma_tag');
  if numel(tag) > 0
    if numel(str) > 0
      str = [str, ' '];
    end
    str = [str, '[', tag, ']'];
  end

  if nargin > 1 && numel(ext) > 0
    if numel(str) > 0
      str = [str, ' - ', ext];
    else
      str = ext;
    end
  end

  if nargout == 0
    set(fig, 'Name', str);
  end

% coma/dirty -- handle dirty flag.
%
%      flag = coma('dirty', obj, flag)
%
% Set or get the dirty flag of widgets.
%
% First argument OBJ is an array of Handle Graphics objects.
% Optional second argument FLAG is the dirty flag.  An empty
%  array or a value of zero means to clear the dirty flag for
%  the specified widgets.  Otherwise, the dirty flag is raised.
%  Argument FLAG is either a scalar or an array with the same
%  number of elements as in OBJ.
%
% If argument FLAG is omitted, return value is the current status
% of the dirty flag for the specified widgets.
function flag = coma_dirty(obj, flag)

  m = numel(obj);
  if nargin > 1
    % Set dirty flag.
    if isempty(flag)
      flag = 0;
    end
    n = numel(flag);
    if m == 1 && n == 1
      setappdata(obj, 'coma_dirty', flag);
    elseif m == n
      for i = 1:m
        setappdata(obj(i), 'coma_dirty', flag(i));
      end
    elseif n == 1
      for i = 1:m
        setappdata(obj(i), 'coma_dirty', flag);
      end
    else
      error('Invalid argument');
    end
  else
    % Get dirty flag.
    if m == 1
      flag = getappdata(obj, 'coma_dirty');
      if isempty(flag)
        flag = 0;
      end
    else
      flag = zeros(size(obj));
      for i = 1:m
        tem = getappdata(obj(i), 'coma_dirty');
        if ~ isempty(tem)
          flag(i) = tem;
        end
      end
    end
  end

% coma/enable -- handle enable flag.
%
%      flag = coma('enable', obj, flag)
%
% Set or get the enable property of widgets.
%
% First argument OBJ is an array of Handle Graphics objects.
% Optional second argument FLAG is the enable flag.  An empty
%  array or a value of zero means to disable the specified widgets.
%  Otherwise, the widgets are enabled.  Argument FLAG is either a
%  scalar or an array with the same number of elements as in OBJ.
%
% If argument FLAG is omitted, return value is the current status
% of the enable property for the specified widgets.
function flag = coma_enable(obj, flag)

  m = numel(obj);
  if nargin > 1
    % Enable/disable widgets.
    str = {'off'; 'on'};
    if isempty(flag)
      flag = 0;
    end
    flag = (flag ~= 0);
    n = numel(flag);
    if n == 1
      set(obj, 'Enable', str{1 + flag});
    elseif m == n
      set(obj, {'Enable'}, str(1 + flag));
    else
      error('Invalid argument');
    end
  else
    % Get enable flag.
    flag = strcmp(get(obj, {'Enable'}), 'on');
  end

% coma/visible -- handle visibility flag.
%
%      flag = coma('visible', obj, flag)
%
% Set or get the visibility property of widgets.
%
% First argument OBJ is an array of Handle Graphics objects.
% Optional second argument FLAG is the visibility flag.  An empty
%  array or a value of zero means to hide the specified widgets.
%  Otherwise, the widgets are shown.  Argument FLAG is either a
%  scalar or an array with the same number of elements as in OBJ.
%
% If argument FLAG is omitted, return value is the current status
% of the visibility property for the specified widgets.
function flag = coma_visible(obj, flag)

  m = numel(obj);
  if nargin > 1
    % Show/hide widgets.
    str = {'off'; 'on'};
    if isempty(flag)
      flag = 0;
    end
    flag = (flag ~= 0);
    n = numel(flag);
    if n == 1
      set(obj, 'Visible', str{1 + flag});
    elseif m == n
      set(obj, {'Visible'}, str(1 + flag));
    else
      error('Invalid argument');
    end
  else
    % Get visibility flag.
    flag = strcmp(get(obj, {'Visible'}), 'on');
  end

% coma/tick -- handle tick mark.
%
%      flag = coma('tick', obj, flag)
%
% Set or get the tick mark property of widgets.
%
% First argument OBJ is an array of Handle Graphics objects.
% Optional second argument FLAG is the tick mark flag.  An empty
%  array or a value of zero means to uncheck/deselect the specified
%  widgets.  Otherwise, the widgets are checked/selected.  Argument
%  FLAG is either a scalar or an array with the same number of
%  elements as in OBJ.
%
% If argument FLAG is omitted, return value is the current status
% of the tick mark property for the specified widgets.
function flag = coma_tick(obj, flag)

  if nargin > 1
    % Set/clear tick mark.
    if isempty(flag)
      flag = false;
    elseif ~ islogical(flag)
      flag = (flag ~= false);
    end
    n = numel(flag);
    if n == 1 || n == numel(obj)
      val = get(obj, {'Min', 'Max'});
      set(obj, {'Value'}, val(:, 1 + flag));
    else
      error('Invalid argument');
    end
  else
    % Query tick mark.
    val = get(obj, {'Value', 'Max'});
    flag = [val{:, 1}] == [val{:, 2}];
    flag = reshape(flag, size(obj));
  end

% coma/hookp -- predicate function.
%
%      p = coma('hookp', obj)
%
% Return true if argument OBJ is a call-back function.
function p = coma_hookp(obj)

  if iscell(obj) && numel(obj) > 0
    obj = obj{1};
  end

  switch class(obj)
   case 'function_handle'
    p = true;
   case 'char'
    p = (mtk('stringp', obj) && numel(obj) > 0);
   otherwise
    p = false;
  end

% coma/run_hook -- invoke a call-back function.
%
%      [...] = coma('run_hook', hook, ...)
%
% Run HOOK with arguments.  If HOOK is empty, do nothing.
%
% A hook is either a function name (a string), a function handle,
% or a cell array.  For the later, the first element has to be
% either a function name or a function handle and the remaining
% elements are additional arguments for the designated function.
function varargout = coma_run_hook(hook, varargin)

  if nargout > 0
    varargout = cell(1, nargout);
  end

  if isempty(hook)
    return;
  end

  switch class(hook)
   case {'char', 'function_handle'}
    hook = {hook};
   case 'cell'
   otherwise
    error('Invalid hook function');
  end

  if numel(varargin) > 0
    hook = [hook(1), varargin, hook(2:end)];
  end

  if nargout == 0
    feval(hook{:});
  else
    [varargout{:}] = feval(hook{:});
  end

% coma/filename -- return file name.
%
%      name = coma('filename', str)
%
% Return the file name part of a file name string, i.e. the file
% name without directory part and without the file name extension.
function name = coma_filename(filename)

  [dir, name, ext] = fileparts(filename);

%% coma.m ends here
