% mico.m -- load graphics image.
%
%      [color, alpha] = mico(object, background)
%
% Return direct color data for displaying a graphics image.
%
% First argument OBJECT is the image object.  If OBJECT is a string,
%  the image data is loaded from file name OBJECT.  If OBJECT is an
%  array of 'uint8' elements, it is treated as the raw image data,
%  that is the same sequence of bytes as if the image is saved in a
%  file.  If OBJECT is a cell array, the Tango 'icon' command is
%  called with the elements of the cell array as arguments and the
%  return value is interpreted as the icon file name.
% If the optional second argument BACKGROUND is given, the
%  transparency information of the image is merged with the color
%  value defined by BACKGROUND.  Argument BACKGROUND is either a color
%  specification or a Handle Graphics object.  If the Handle Graphics
%  object is the root object (that is zero), the default background
%  color of 'uicontrol' objects is utilized.
%
% Example:
%
% f = figure('Color', get(0, 'defaultUIControlBackgroundColor'));
% h = uicontrol('Parent', f, ...
%               'Style', 'pushbutton', ...
%               'CData', mico({'go-home', 24}, 0), ...
%               'Units', 'pixels', ...
%               'Position', [8, 8, 40, 40]);
%
% See also: mtk, tango

%% mico.m --- the preceding comment is the documentation string.

% Copyright (C) 2008 Ralph Schleicher

% This file is part of MTk, the Matlab widget toolkit.

% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions
% are met:
%
%    * Redistributions of source code must retain the above copyright
%      notice, this list of conditions and the following disclaimer.
%
%    * Redistributions in binary form must reproduce the above copyright
%      notice, this list of conditions and the following disclaimer in
%      the documentation and/or other materials provided with the
%      distribution.
%
%    * Neither the name of the copyright holder nor the names of its
%      contributors may be used to endorse or promote products derived
%      from this software without specific prior written permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
% "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
% FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
% COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
% INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
% CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
% LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
% ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.

%% Code:

% Program entry point.
function [col, alpha] = mico(obj, back)

  % Check arguments.
  if nargin < 1
    error(nargchk(1, 2, nargin));
  end
  if nargin < 2
    back = [];
  end

  % Examine image object.
  switch class(obj)
   case 'cell'
    filename = tango('icon', obj{:});
    if isempty(filename)
      error('Failed to lookup icon');
    end
    filedata = [];
    loadname = filename;
   case 'char'
    filename = obj;
    filedata = [];
    loadname = mico_find(obj);
   case 'uint8'
    filename = '<built-in image>';
    filedata = obj(:);
    if isempty(filedata)
      error('Invalid image object');
    end
   otherwise
    error('Invalid image object');
  end

  % Load image data.
  try
    if isempty(filedata)
      [col, map, alpha] = mico_load(loadname);
    else
      loadname = mico_save(tempname, filedata);
      [col, map, alpha] = mico_load(loadname);
      delete(loadname);
    end
  catch
    error('Can not load image');
  end

  m = size(col, 1);
  n = size(col, 2);

  % Convert image data to color data.
  switch class(col)
   case {'double', 'single'}
    f = 1;
   case 'logical'
    f = 1;
   case 'uint8'
    f = 255;
   case 'uint16'
    f = 65535;
   case 'uint32'
    f = 4294967295;
   otherwise
    error('Invalid image data type');
  end

  col = double(col);
  if isempty(map)
    % Direct-color.
    if f ~= 1
      col = col ./ f;
      % Fix rounding errors.
      col(col < 0) = 0;
      col(col > 1) = 1;
    end
    % Expand gray-scale images.
    if ndims(col) == 2
      col = cat(3, col, col, col);
    end
  else
    % Pseudo-color.
    col = col(:);
    if any(col == 0)
      col = col + 1;
    end
    col = reshape([map(col, 1); map(col, 2); map(col, 3)], m, n, 3);
  end

  alpha = double(alpha);
  if isempty(alpha)
    % Fully opaque.
    alpha = 1;
  elseif f ~= 1
    alpha = alpha ./ f;
    % Fix rounding errors.
    alpha(alpha < 0) = 0;
    alpha(alpha > 1) = 1;
  end

  % Apply background transparency.
  if isempty(back) || all(alpha(:) == 1)
    return;
  end

  % Examine background color.
  if numel(back) == 1 && ishandle(back)
    switch get(back, 'Type')
     case 'root'
      back = get(back, 'defaultUIControlBackgroundColor');
     case 'figure'
      back = get(back, 'Color');
     case 'axes'
      tem = get(back, 'Color');
      if ischar(tem) && strcmp(tem, 'none')
	back = get(get(back, 'Parent'), 'Color');
      else
        back = tem;
      end
     case 'uicontrol'
      back = get(back, 'BackgroundColor');
     otherwise
      error('Invalid Handle Graphics object');
    end
  end

  if ischar(back)
    switch back
     case {'y', 'yellow'}
      back = [1, 1, 0];
     case {'m', 'magenta'}
      back = [1, 0, 1];
     case {'c', 'cyan'}
      back = [0, 1, 1];
     case {'r', 'red'}
      back = [1, 0, 0];
     case {'g', 'green'}
      back = [0, 1, 0];
     case {'b', 'blue'}
      back = [0, 0, 1];
     case {'w', 'white'}
      back = [1, 1, 1];
     case {'k', 'black'};
      back = [0, 0, 0];
     otherwise
      error('Invalid background color name');
    end
  else
    switch class(back)
     case {'double', 'single'}
      if any(imag(back) ~= 0 | back < 0 | back > 1)
        error('Invalid background color value');
      end
      back = double(back);
     case 'logical'
      back = double(back);
     case 'uint8'
      back = double(back) ./ 255;
     case 'uint16'
      back = double(back) ./ 65535;
     case 'uint32'
      back = double(back) ./ 4294967295;
     otherwise
      error('Invalid background color data type');
    end
    switch numel(back)
     case 1
      back = repmat(back, 1, 3);
     case 3
      back = reshape(back, 1, 3);
     otherwise
      if ndims(back) == 2
        back = cat(3, back, back, back);
      end
      if ndims(back) ~= 3 || any(size(back) ~= size(col))
        error('Invalid background color dimensions');
      end
    end
  end

  % Merge background color.
  switch ndims(back)
   case 2
    col(:, :, 1) = col(:, :, 1) .* alpha + back(1) .* (1 - alpha);
    col(:, :, 2) = col(:, :, 2) .* alpha + back(2) .* (1 - alpha);
    col(:, :, 3) = col(:, :, 3) .* alpha + back(3) .* (1 - alpha);
   case 3
    a = cat(3, alpha, alpha, alpha);
    col = col .* a + back .* (1 - a);
  end

% Lookup image file name.
function loadname = mico_find(filename)

  % Resolve system dependencies.
  if ispc
    filename = strrep(filename, '/', filesep);
  end

  % Try FILENAME as is.
  if mtk('implicitp', filename)
    loadname = fullfile('.', filename);
    if exist(loadname, 'file')
      return;
    end
  else
    % File name is absolute or explicit relative.
    loadname = filename;
    return;
  end

  % Look into various directories.
  icon_dir = tango('icon_dir');
  for i = 1:numel(icon_dir)
    loadname = fullfile(icon_dir{i}, filename);
    if exist(loadname, 'file')
      return;
    end
  end

  % If all fails, return FILENAME as is.
  loadname = fullfile('.', filename);

% Load image data from file.
function [col, map, alpha] = mico_load(filename)

  try
    [col, map, alpha] = imread(filename);
  catch
    [col, map] = imread(filename);
    % No alpha channel.
    alpha = [];
  end

% Save raw image data to file.
function filename = mico_save(filename, filedata)

  [h, err] = fopen(filename, 'w');
  if h == -1
    error(err);
  end

  if fwrite(h, filedata) < numel(filedata)
    err = 'Incomplete write';
  end

  if fclose(h) == 0
    h = -1;
  elseif isempty(err)
    err = 'Can not close file';
  end

  if isempty(err)
    return;
  end

  if h == -1
    delete(filename);
  end

  error(err);

%% mico.m ends here
