% mico_dump -- dump graphics image.
%
%      mico_dump(filename, output)
%
% Generate Matlab code for a built-in icon.
%
% First argument FILENAME is the icon file name to be dumped.
% Optional second argument OUTPUT is the output file name.
%  If OUTPUT is omitted, write to standard output.
%
% Example:
%
%      % Create built-in icon, that is a Matlab function
%      % returning the image file contents.
%      mico_dump('an_icon.png', 'an_icon.m');
%
%      % Now load it via the 'mico' command.
%      image(mico(an_icon));
%
% See also: mico

%% mico_dump.m --- the preceding comment is the documentation string.

% Copyright (C) 2008 Ralph Schleicher

% This file is part of MTk, the Matlab widget toolkit.

% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions
% are met:
%
%    * Redistributions of source code must retain the above copyright
%      notice, this list of conditions and the following disclaimer.
%
%    * Redistributions in binary form must reproduce the above copyright
%      notice, this list of conditions and the following disclaimer in
%      the documentation and/or other materials provided with the
%      distribution.
%
%    * Neither the name of the copyright holder nor the names of its
%      contributors may be used to endorse or promote products derived
%      from this software without specific prior written permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
% "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
% FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
% COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
% INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
% CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
% LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
% ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.

%% Code:

% Program entry point.
function mico_dump(filename, output)

  % Check number of arguments.
  if nargin < 1
    error(nargchk(1, 2, nargin));
  end
  if nargin < 2
    output = [];
  end

  % Get image file information.
  info = imfinfo(filename);
  if numel(info) > 1
    error('Multi-image files are not supported');
  end

  % Read file contents.
  h = fopen(filename, 'rb');
  if h == -1
    error(sprintf('%s: can not open file', filename));
  end

  try
    ch = fread(h);
    if ~ feof(h)
      error(sprintf('%s: incomplete read', fopen(h)));
    end
    if fclose(h) == -1
      error(sprintf('%s: can not close file', fopen(h)));
    end
  catch
    fclose(h);
    if exist('rethrow', 'builtin')
      rethrow(lasterror);
    else
      error(lasterr);
    end
  end

  % Dump file contents.
  if isempty(output)
    h = 1;
  else
    h = fopen(output, 'wt');
    if h == -1
      error(sprintf('%s: can not open file', output));
    end
  end

  [dir, base, ext] = fileparts(filename);
  fprintf(h, '%% Return built-in image data.\n');
  fprintf(h, 'function [filedata, filename] = %s\n', base);
  fprintf(h, '\n');
  fprintf(h, '  hex = [''');
  for i = 1:numel(ch)
    fprintf(h, '%02x', ch(i));
    if i < numel(ch) && rem(i, 32) == 0
      fprintf(h, ''', ...\n');
      fprintf(h, '         ''');
    end
  end
  fprintf(h, '''];\n');
  fprintf(h, '\n');
  fprintf(h, '  ind = 1:2:numel(hex);\n');
  fprintf(h, '  hex = [hex(ind); hex(ind + 1)]'';\n');
  fprintf(h, '\n');
  fprintf(h, '  filedata = uint8(hex2dec(hex));\n');
  fprintf(h, '  filename = ''%s'';\n', strrep([base, ext], '''', ''''''));

  if h ~= 1
    if fclose(h) == -1
      error(sprintf('%s: can not close file', fopen(h)));
    end
  end

%% mico_dump.m ends here
